@@ -1,7 +1,7 @@
 .onAttach <- function(...) {
   # packageStartupMessage("Important changes: constructors legend() and title() have been replaced by amLegend() and amTitle().")
   packageStartupMessage("Full amcharts.js API available using amChartsAPI()")
-  packageStartupMessage("Look at rAmCharts::runExamples() & http://datastorm-open.github.io/introduction_ramcharts/")
+  packageStartupMessage("Look at rAmCharts::runExamples() & https://datastorm-open.github.io/introduction_ramcharts/")
   packageStartupMessage("Bug report or feed back on https://github.com/datastorm-open/rAmCharts")
   # packageStartupMessage("\nRemove this message with the following command line: suppressPackageStartupMessages(library(rAmCharts))")
 }
